﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.BusinessLogicsContracts;
using FishFactoryContracts.SearchModels;
using FishFactoryContracts.StoragesContracts;
using FishFactoryContracts.ViewModels;
using FishFactoryDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FishFactoryBusinessLogic.BusinessLogics
{
	public class ShopLogic : IShopLogic
	{
		private readonly ILogger _logger;
		private readonly IShopStorage _shopStorage;
		public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
		{
			_logger = logger;
			_shopStorage = shopStorage;
		}
		public List<ShopViewModel>? ReadList(ShopSearchModel? model)
		{
			_logger.LogInformation("ReadList. ShopName: {ShopName}. Id: {Id}",
				model?.ShopName, model?.Id);
			var list = model == null ? _shopStorage.GetFullList() :
				_shopStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public ShopViewModel? ReadElement(ShopSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. ShopName: {ShopName}. Id: {Id}",
				model.ShopName, model.Id);
			var element = _shopStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(ShopBindingModel model)
		{
			CheckModel(model);
			if (_shopStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(ShopBindingModel model)
		{
			CheckModel(model);
			if (_shopStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(ShopBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id: {Id}", model.Id);
			if (_shopStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(ShopBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.ShopName))
			{
				throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
			}
			_logger.LogInformation("Shop. ShopName:{0}.Address:{1}. Id: {2}",
				model.ShopName, model.Address, model.Id);
			var element = _shopStorage.GetElement(new ShopSearchModel
			{
				ShopName = model.ShopName
			});
			if (element != null && element.Id != model.Id && element.ShopName == model.ShopName)
			{
				throw new InvalidOperationException("Магазин с таким названием уже есть");
			}
		}

		public bool AddCannedInShop(ShopSearchModel model, ICannedModel Canned, int count)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (count <= 0)
			{
				throw new ArgumentException("Количество консерв должно быть больше 0", nameof(count));
			}
			_logger.LogInformation("AddCannedInShop. ShopName:{ShopName}.Id:{ Id}",
				model.ShopName, model.Id);
			var element = _shopStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("AddCannedInShop element not found");
				return false;
			}
			_logger.LogInformation("AddCannedInShop find. Id:{Id}", element.Id);

			if (element.ListCanned.TryGetValue(Canned.Id, out var pair))
			{
				element.ListCanned[Canned.Id] = (Canned, count + pair.Item2);
				_logger.LogInformation(
					"AddCannedInShop. Added {count} {Canned} to '{ShopName}' shop",
					count, Canned.CannedName, element.ShopName);
			}
			else
			{
				element.ListCanned[Canned.Id] = (Canned, count);
				_logger.LogInformation(
					"AddCannedInShop. Added {count} new Canned {Canned} to '{ShopName}' shop",
					count, Canned.CannedName, element.ShopName);
			}
			_shopStorage.Update(new()
			{
				Id = element.Id,
				Address = element.Address,
				ShopName = element.ShopName,
				DateOpening = element.DateOpening,
				ListCanned = element.ListCanned
			});
			return true;
		}
	}
}
